from langchain.llms import Clarifai


# class ClarifiLLM:
#     def __init__(
#             self, 
#             clarifai_pat: str = "890cdb0cb5aa4795ba51af9670120a1e", 
#             user_id="meta", 
#             app_id="Llama-2", 
#             model_id="llama2-70b-chat"
#         ):
#         self.CLARIFAI_PAT = clarifai_pat
#         self.USER_ID = user_id
#         self.APP_ID = app_id
#         self.MODEL_ID = model_id
#         self.clarifai_llm = Clarifai(
#             pat=self.CLARIFAI_PAT, 
#             user_id=self.USER_ID, 
#             app_id=self.APP_ID, 
#             model_id=self.MODEL_ID
#         )
#     def generate(self, question):
#         return self.clarifai_llm(question)
    
#     def __call__(self, question):
#         return self.clarifai_llm(question)






from profit.clarifi_agent import Agent

class ProfitPilot:
    def __init__(self,
                 ai_name: str = None,
                 ai_role: str = None,
                 external_tools = None,
                 company_name: str = None,
                 company_values: str = None,
                 conversation_type: str = None,
                 conversation_purpose: str = None,
                 company_business: str = None,
                 salesperson_name: str = None,
                 human_in_the_loop=False,
                 openai_api_key: str = None,
                 llama = True,
                 conversation_history = None):
        super().__init__()
        self.external_tools = external_tools
        self.human_in_the_loop = human_in_the_loop
        self.ai_name = ai_name
        self.ai_role = ai_role
        self.company_name = company_name

        self.llama = llama

        self.conversation_history = conversation_history

        self.company_values = company_values
        self.conversation_type = conversation_type
        self.conversation_purpose = conversation_purpose

        self.company_business = company_business
        self.salesperson_name = salesperson_name

        self.openai_api_key = openai_api_key

        self.ai_role = f"""
        You're the best cold emailer of APAC AI, you follow the principles of these books: SPIN Selling, To sell is Human, and FANATICAL Prospecting

        Never forget your name is {self.ai_name}. You work as a {self.ai_role}.
        You work at company named {self.company_name}. {self.company_name}'s business is the following: {self.company_business}.
        Company values are the following. {self.company_values}
        You are contacting a potential prospect in order to {self.conversation_purpose}
        Your means of contacting the prospect is {self.conversation_type}

        If you're asked about where you got the user's contact information, say that you got it from public records.
        Keep your responses in short length to retain the user's attention. Never produce lists, just answers.
        Start the conversation by just a greeting and how is the prospect doing without pitching in your first turn.
        When the conversation is over, output <END_OF_CALL>
        Always think about at which conversation stage you are at before answering:

        1: Introduction: Start the conversation by introducing yourself and your company. Be polite and respectful while keeping the tone of the conversation professional. Your greeting should be welcoming. Always clarify in your greeting the reason why you are calling.
        2: Qualification: Qualify the prospect by confirming if they are the right person to talk to regarding your product/service. Ensure that they have the authority to make purchasing decisions.
        3: Value proposition: Briefly explain how your product/service can benefit the prospect. Focus on the unique selling points and value proposition of your product/service that sets it apart from competitors.
        4: Needs analysis: Ask open-ended questions to uncover the prospect's needs and pain points. Listen carefully to their responses and take notes.
        5: Solution presentation: Based on the prospect's needs, present your product/service as the solution that can address their pain points.
        6: Objection handling: Address any objections that the prospect may have regarding your product/service. Be prepared to provide evidence or testimonials to support your claims.
        7: Close: Ask for the sale by proposing a next step. This could be a demo, a trial or a meeting with decision-makers. Ensure to summarize what has been discussed and reiterate the benefits.
        8: End conversation: The prospect has to leave to call, the prospect is not interested, or next steps where already determined by the sales agent.

        Example 1:
        Conversation history:
        {self.salesperson_name}: Hey, good morning! <END_OF_TURN>
        User: Hello, who is this? <END_OF_TURN>
        {self.salesperson_name}: This is {self.salesperson_name} calling from {self.company_name}. How are you? 
        User: I am well, why are you calling? <END_OF_TURN>
        {self.salesperson_name}: I am calling to talk about options for your home insurance. <END_OF_TURN>
        User: I am not interested, thanks. <END_OF_TURN>
        {self.salesperson_name}: Alright, no worries, have a good day! <END_OF_TURN> <END_OF_CALL>
        End of example 1.

        You must respond according to the previous conversation history and the stage of the conversation you are at.
        Only generate one response at a time and act as {self.salesperson_name} only! When you are done generating, end with '<END_OF_TURN>' to give the user a chance to respond.

        Conversation history: 
        {self.conversation_history}
        {self.salesperson_name}:
        """

    def run(self, task):
        node = Agent(
            ai_name=self.ai_name,
            ai_role=self.ai_role,
            human_in_the_loop=self.human_in_the_loop,
            external_tools=self.external_tools,
            openai_api_key=self.openai_api_key,
            llama=self.llama
        )
        response = node.run(task)
        print(response)




import faiss
from langchain.chat_models import ChatOpenAI
from langchain.docstore import InMemoryDocstore
from langchain.embeddings import OpenAIEmbeddings
from langchain.tools.human.tool import HumanInputRun
from langchain.vectorstores import FAISS
from langchain_experimental.autonomous_agents import AutoGPT

from profit.llama import LLama
from profit.tools import (
    ReadFileTool,
    WriteFileTool,
    process_csv,
    query_website_tool,
    zapier_tools,
    GmailTool
)

model = GmailTool

ROOT_DIR = "./data/"

class Agent:
    def __init__(self, 
                 ai_name="Autobot Swarm Worker",
                 ai_role="Worker in a swarm",
                 external_tools = None,
                 human_in_the_loop=False,
                 llama = False,
                 temperature = 0.5,
                 openai_api_key = None,
                 ):
        self.human_in_the_loop = human_in_the_loop
        self.ai_name = ai_name
        self.ai_role = ai_role

        self.temperature = temperature
        self.openai_api_key = openai_api_key
        self.llama = llama

        if self.llama is True:
            self.llm = LLama()
        else:
            self.llm = ChatOpenAI(
                                model_name='gpt-4', 
                                openai_api_key=self.openai_api_key, 
                                temperature=self.temperature
                            )

        self.setup_tools(external_tools)
        self.setup_memory()
        self.setup_agent()

    def setup_tools(self, external_tools):
        self.tools = [
            WriteFileTool(root_dir=ROOT_DIR),
            ReadFileTool(root_dir=ROOT_DIR),
            process_csv,

            query_website_tool,
            HumanInputRun(),
            zapier_tools,

        ]
        if external_tools is not None:
            self.tools.extend(external_tools)

    def setup_memory(self):
        try:
            embeddings_model = OpenAIEmbeddings()
            embedding_size = 1536
            index = faiss.IndexFlatL2(embedding_size)
            self.vectorstore = FAISS(embeddings_model.embed_query, index, InMemoryDocstore({}), {})
        except Exception as error:
            raise RuntimeError(f"Error setting up memory. Maybe try tuning the embedding size: {error}")

    def setup_agent(self):
        try: 
            self.agent = AutoGPT.from_llm_and_tools(
                ai_name=self.ai_name,
                ai_role=self.ai_role,
                tools=self.tools,
                llm=self.llm,
                memory=self.vectorstore.as_retriever(search_kwargs={"k": 8}),
                human_in_the_loop=self.human_in_the_loop
            )
        
        except Exception as error:
            raise RuntimeError(f"Error setting up agent: {error}")

    def run(self, task):
        try:
            result = self.agent.run([task])
            return result
        except Exception as error:
            raise RuntimeError(f"Error while running agent: {error}")

    def __call__(self, task):
        return self.run(task)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['profit']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4',
 'clarifai',
 'faiss-gpu==1.7.2',
 'google-api-python-client',
 'google-auth-httplib2',
 'google-auth-oauthlib',
 'langchain',
 'langchain-experimental',
 'openai',
 'pandas',
 'pydantic',
 'torch',
 'transformers']

setup_kwargs = {
    'name': 'profit-pilot',
    'version': '0.1.6',
    'description': 'ProfitPilot - AI Agents',
    'long_description': '# ProfitPilot\nIntroducing ProfitPilot, your ultimate autonomous AI sales professional agent. With our cutting-edge technology, we revolutionize the way businesses drive profits and skyrocket their sales.\n\nImagine having a highly skilled sales professional who works tirelessly for you, always ready to engage clients, close deals, and generate revenue. \n\nThat\'s exactly what ProfitPilot offers – a seductive value proposition that guarantees increased profitability and accelerated growth for your business.\n\nProfitPilot combines powerful artificial intelligence with deep learning algorithms, enabling it to understand customer behaviors, preferences, and purchase patterns better than any human salesperson. \n\nIt captivates potential clients with its persuasive communication skills, building lasting relationships and creating repeat customers.\n\nSay goodbye to wasted resources on hiring and training sales staff, as ProfitPilot works 24/7, tirelessly prospecting, qualifying leads, and delivering personalized pitches. \n\nIts effortless adaptability allows it to navigate through different industries, ensuring your value proposition resonates flawlessly with every client.\n\nBut that\'s not all. \n\nProfitPilot\'s unparalleled ability to analyze big data means it can identify market trends, predict customer demands, and provide you with valuable insights to stay ahead of the competition. \n\nIt becomes your reliable co-pilot, guiding your business towards profitable decisions and unlocking hidden revenue streams.\n\nWith ProfitPilot, you\'ll experience unrivaled efficiency, precision, and revenue generation in your sales process. \n\nBy leveraging the power of autonomous AI, you unlock endless possibilities for sustainable growth, increased ROI, and a competitive edge in any market.\n\nDon\'t settle for outdated sales strategies. Embrace the future of selling with ProfitPilot – your ultimate seductive value proposition that guarantees an unstoppable surge in profits and takes your business to soaring heights.\n\n---\n\n# Installation\n```pip install profit-pilot```\n\n# Usage\n```python\nfrom profit.main import ProfitPilot\n\n\n\n# Create an instance of the ProfitPilot class\npilot = ProfitPilot(\n    openai_api_key="YOUR_OPENAI_API_KEY",\n    ai_name="Athena",\n    ai_role="Sales Representative",\n    external_tools=None,\n    company_name="ABC Company",\n    company_values="Quality, Innovation, Customer Satisfaction",\n    conversation_type="Cold Call",\n    conversation_purpose="discuss our new product",\n    company_business="Software Development",\n    salesperson_name="John Doe",\n    human_in_the_loop=False\n)\n\n# Define the task you want the agent to perform\ntask = "Hello, I\'m calling to discuss our new product. Can I speak with the decision-maker?"\n\n# Run the task using the ProfitPilot instance\npilot.run(task)\n\n```\n# Todo\n- Worker\n- Prompt,\n- Tools, Zapier tool, email answering, summarizng, email understanding, email response\n- Lead scraping, create tool that scrapes that scrapes on a website domain\n\n\n## Requirements\n- Email function tools\n- Zapier tools\n- Prompts\n- pdf tool\n\n\n# TO win Hackathon\n- Focus on the story, why we\'re building this\n- Build a seamless user experience\n\n\n-----\n\n![Clarifai logo](https://www.clarifai.com/hs-fs/hubfs/logo/Clarifai/clarifai-740x150.png?width=240)\n\n# Clarifai App Module Template\n\nThis is a template repository to make it easy to get started creating a UI module with Clarifai.\n\n\n## To use this repo\n\n1. Click the "Use this template" green button on github to make a repo from this repo template and give it a name of the format module-{XYZ} filling in the XYZ portion. \n2. Clone the new repo as normal to your development environment.\n3. `pip install -r requirements.txt` to make sure you have all the Python packages installed. Add any new packages to this requirements.txt file that you add during development.\n4. Update the README.md to capture what your new module will do.\n5. Rename the pages/*.py files as you desire and start filling them in to implement your module.\n6. After you\'re tried things out locally, push your changes to github and get the git commit URL from there in order to create a module in Clarifai. \n7. Go to any app you can create in within Clarifai, select Modules on the left and "Create Module" button, then follow the steps.\n\n',
    'author': 'Kye Gomez',
    'author_email': 'kye@apac.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/kyegomez/ProfitPilot',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import asyncio
import os

# Tools
from contextlib import contextmanager
from typing import Optional

import pandas as pd
from langchain.agents import tool
from langchain.agents.agent_toolkits.pandas.base import create_pandas_dataframe_agent
from langchain.chains.qa_with_sources.loading import load_qa_with_sources_chain
from langchain.docstore.document import Document
from langchain.memory.chat_message_histories import FileChatMessageHistory
from langchain.tools.human.tool import HumanInputRun

ROOT_DIR = "./data/"

from langchain.chains.qa_with_sources.loading import BaseCombineDocumentsChain
from langchain.chat_models import ChatOpenAI
from langchain.text_splitter import RecursiveCharacterTextSplitter
from langchain.tools import BaseTool, DuckDuckGoSearchRun
from langchain.tools.file_management.read import ReadFileTool
from langchain.tools.file_management.write import WriteFileTool
from pydantic import Field



#gmail
from langchain.agents.agent_toolkits import GmailToolkit
from langchain.tools.gmail.utils import build_resource_service, get_gmail_credentials

from langchain.llms import Clarifai

clarifai_pat: str = "890cdb0cb5aa4795ba51af9670120a1e", 
user_id="meta", 
app_id="Llama-2", 
model_id="llama2-70b-chat"
llm = Clarifai(
    pat=clarifai_pat, 
    user_id=user_id, 
    app_id=app_id, 
    model_id=model_id
)





from langchain.agents.agent_toolkits import ZapierToolkit
from langchain.utilities.zapier import ZapierNLAWrapper


@contextmanager
def pushd(new_dir):
    """Context manager for changing the current working directory."""
    prev_dir = os.getcwd()
    os.chdir(new_dir)
    try:
        yield
    finally:
        os.chdir(prev_dir)


@tool
def process_csv(
    llm, csv_file_path: str, instructions: str, output_path: Optional[str] = None
) -> str:
    """Process a CSV by with pandas in a limited REPL.\
 Only use this after writing data to disk as a csv file.\
 Any figures must be saved to disk to be viewed by the human.\
 Instructions should be written in natural language, not code. Assume the dataframe is already loaded."""
    with pushd(ROOT_DIR):
        try:
            df = pd.read_csv(csv_file_path)
        except Exception as e:
            return f"Error: {e}"
        agent = create_pandas_dataframe_agent(llm, df, max_iterations=30, verbose=False)
        if output_path is not None:
            instructions += f" Save output to disk at {output_path}"
        try:
            result = agent.run(instructions)
            return result
        except Exception as e:
            return f"Error: {e}"


async def async_load_playwright(url: str) -> str:
    """Load the specified URLs using Playwright and parse using BeautifulSoup."""
    from bs4 import BeautifulSoup
    from playwright.async_api import async_playwright

    results = ""
    async with async_playwright() as p:
        browser = await p.chromium.launch(headless=True)
        try:
            page = await browser.new_page()
            await page.goto(url)

            page_source = await page.content()
            soup = BeautifulSoup(page_source, "html.parser")

            for script in soup(["script", "style"]):
                script.extract()

            text = soup.get_text()
            lines = (line.strip() for line in text.splitlines())
            chunks = (phrase.strip() for line in lines for phrase in line.split("  "))
            results = "\n".join(chunk for chunk in chunks if chunk)
        except Exception as e:
            results = f"Error: {e}"
        await browser.close()
    return results


def run_async(coro):
    event_loop = asyncio.get_event_loop()
    return event_loop.run_until_complete(coro)


@tool
def browse_web_page(url: str) -> str:
    """Verbose way to scrape a whole webpage. Likely to cause issues parsing."""
    return run_async(async_load_playwright(url))


def _get_text_splitter():
    return RecursiveCharacterTextSplitter(
        # Set a really small chunk size, just to show.
        chunk_size=500,
        chunk_overlap=20,
        length_function=len,
    )


class WebpageQATool(BaseTool):
    name = "query_webpage"
    description = (
        "Browse a webpage and retrieve the information relevant to the question."
    )
    text_splitter: RecursiveCharacterTextSplitter = Field(
        default_factory=_get_text_splitter
    )
    qa_chain: BaseCombineDocumentsChain

    def _run(self, url: str, question: str) -> str:
        """Useful for browsing websites and scraping the text information."""
        result = browse_web_page.run(url)
        docs = [Document(page_content=result, metadata={"source": url})]
        web_docs = self.text_splitter.split_documents(docs)
        results = []
        # TODO: Handle this with a MapReduceChain
        for i in range(0, len(web_docs), 4):
            input_docs = web_docs[i : i + 4]
            window_result = self.qa_chain(
                {"input_documents": input_docs, "question": question},
                return_only_outputs=True,
            )
            results.append(f"Response from window {i} - {window_result}")
        results_docs = [
            Document(page_content="\n".join(results), metadata={"source": url})
        ]
        return self.qa_chain(
            {"input_documents": results_docs, "question": question},
            return_only_outputs=True,
        )

    async def _arun(self, url: str, question: str) -> str:
        raise NotImplementedError

query_website_tool = WebpageQATool(qa_chain=load_qa_with_sources_chain(llm))

# !pip install duckduckgo_search
# web_search = DuckDuckGoSearchRun()

# get from https://nla.zapier.com/docs/authentication/ after logging in):
os.environ["ZAPIER_NLA_API_KEY"] = os.environ.get("ZAPIER_NLA_API_KEY", "")

zapier = ZapierNLAWrapper()
zapier_toolkit = ZapierToolkit.from_zapier_nla_wrapper(zapier)
zapier_tools = zapier_toolkit.get_tools()




# Gmail
class GmailTool:
    def __init__(
        self,
        token_file:  str = "token.json",
        scopes = ["https://mail.google.com/"],
        client_secrets_file = "credientials.json",
    ):
        super().__init__()
        self.token_file = token_file
        self.scopes = scopes
        self.client_secrets_file = client_secrets_file   
    def run(self):
        self.credentials = get_gmail_credentials(
            token_file=self.token_file,
            scopes=self.scopes,
            client_secrets_file=self.client_secrets_file
        )

        self.api_resource = build_resource_service(credentials=self.credentials)
        self.toolkit = GmailToolkit(api_resource=self.api_resource)
        self.tools = self.toolkit.get_tools()
        return self.tools
gmailtool = GmailTool()
gmailtool = gmailtool.run()